/*
 * commands.c
 *
 * Copyright 2000, Chris Pinkham.
 * All Rights Reserved.
 */

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "config.h"
#include "commands.h"
#include "gyach.h"
#include "yahoochat.h"
#include "yahoolib.h"

#include "nix.h"

/*
#define FORK_EXEC
*/

extern SESSION *GlobalSessionHook;

/* fixme: this is in main.c */
void append_to_textbox(GtkWidget * window, GtkWidget * textbox, char *text);
int chatOut( SESSION *sess, char *cmd );

char *find_whitespace( char *str ) {
	char *ptr = str;

	while(( *ptr ) && ( ! isspace( *ptr )))
		ptr++;

	return( ptr );
}

char *skip_whitespace( char *str ) {
	char *ptr = str;

	while(( *ptr ) && ( isspace( *ptr )))
		ptr++;

	return( ptr );
}

void cmd_join( char *room ) {
	/* format is "join roomname" */
	send_yahoo_text_command( "join ", room );
}

void cmd_goto( char *user ) {
    u_char  buf[4096];
    u_char  *ptr = create_yahoo_header( buf , PKT_OUT_GOTO );

    ptr = append_str( ptr , user );

    send_yahoo_packet( buf, ptr );
}

void cmd_follow( char *user ) {
	char buf[129] = "";
	int r;

	if ( *user ) {
		/* check to make sure we don't want to display who we're following */
		if ( strcmp( "?", user )) {
			/* make sure they don't try to follow themself */
			if ( !strcmp( user, GlobalSessionHook->user )) {
				sprintf( buf,
					"%sYou can't follow yourself!%s\n",
					YAHOO_COLOR_RED, YAHOO_COLOR_BLACK );
				append_to_textbox( chat_window, chat_text, buf );
				return;
			}

			r = find_user_row( user );
			if ( r < 0 ) {
				sprintf( buf,
					"User %s%s%s is not in this room, can not follow.\n",
					YAHOO_COLOR_RED, user, YAHOO_COLOR_BLACK );
				append_to_textbox( chat_window, chat_text, buf );
				return;
			}

			if ( follow_user ) {
				free( follow_user );
			}
			follow_user = strdup( user );
		}
	} else {
		if ( follow_user ) {
			free( follow_user );
		}
		follow_user = NULL;
	}

	if ( follow_user ) {
		sprintf( buf, "Following user: %s%s%s\n",
			YAHOO_COLOR_RED, follow_user, YAHOO_COLOR_BLACK );
	} else {
		sprintf( buf, "%sNot currently following anyone.%s\n",
			YAHOO_COLOR_RED, YAHOO_COLOR_BLACK );
	}
	append_to_textbox( chat_window, chat_text, buf );
}

void cmd_invite( char *user ) {
    u_char  buf[4096];
    u_char  *ptr = create_yahoo_header( buf , PKT_OUT_INVITE );

    ptr = append_str( ptr , user );

    send_yahoo_packet( buf, ptr );
}

void cmd_clear() {
	/* fixme */
}

void cmd_tell( char *args ) {
	char *ptr = args;

	/* format is "tell dummytext\001user\001message" */
	ptr = find_whitespace( ptr );
	*ptr = '\001';

	send_yahoo_text_command( "tell tell\001", args );
}

void cmd_away( char *msg ) {
	char buf[31];

	/* if they didn't specify any message, use "Be Right Back" */
	if ( ! msg[0] )
		strcpy( msg, "\x1" );

	if (( msg[0] <= 9 ) ||
		( msg[0] == 11 ) ||
		( msg[0] == 12 )) {
		my_status = msg[0];
		sprintf( buf, "away away\300\200%d\300\200", (int)msg[0] );
		send_yahoo_text_command( buf, "" );
	} else {
		my_status = 10;
		send_yahoo_text_command( "away away\300\200" "10\300\200", msg );
	}
}

void cmd_auto_away( char *value ) {
	char buf[129];

	if ( ! value[0] )
		return;

	auto_away= atoi( value );

	if ( ! auto_away)
		auto_away = 15;

	auto_away_time = time(NULL) + ( auto_away * 60 );

	write_config();

	sprintf( buf, "*** Auto-Away timer set to %d minute(s) ***\n", auto_away );
	append_to_textbox( chat_window, chat_text, buf );
}

/* fixme, possibly use 0x42 the "think" code if can figure it out */
void cmd_think( char *msg ) {
    u_char  buf[4096];
    u_char  *ptr = create_yahoo_header( buf , PKT_OUT_EMOTE );

	ptr = append_str( ptr, "emote\001" );
	ptr = append_str( ptr, ". o O (" );
    ptr = append_str( ptr, msg );
	ptr = append_str( ptr, ")" );

    send_yahoo_packet( buf, ptr );
}

void cmd_roll( char *arg ) {
	/* fixme */
}

void cmd_help( char *arg ) {
	char buf[] = {
		"\n"
		"-----------------------------------------\n"
		"Gyach v" VERSION
			" - Copyright 2000 Chris T. Pinkham, ALL RIGHTS RESERVED\n"
		"\n"
		"Chat Commands:\n"
		"\t/join [room]\t\tjoin the room\n"
		"\t/invite [user]\t\tinvite the user to the current room\n"
		"\t/goto [user]\t\tgoto the user\n"
		"\t/away [msg]\t\tgo away with custom auto-away message\n"
		"\t/autoaway [min]\t\tset auto-away timer to 'min' minutes\n"
		"\t/follow [user]\t\tfollow user\n"
		"\t/follow\t\tun-follow user\n"
		"\t/ignore [user]\t\tignore/un-ignore user\n"
		"\t/ignore\t\tlist all users currently being ignored\n"
		"\t/tell [user] [text]\tsend PM to user\n"
		"\t/think [text]\t\t. o O( think text )\n"
		"\t/exec [command]\t\texecute command and post results\n"
		"\t/url\t\tpaste current Gyach version (" VERSION ") and url\n"
		"\t/quit\n"
		"-----------------------------------------\n"
		"\n" };
	append_to_textbox( chat_window, chat_text, buf );
}

void cmd_url( char *url ) {
	char buf[1025] = "";

	if ( *url ) {
		strcat( buf, YAHOO_STYLE_URLON );
		strcat( buf, url );
		strcat( buf, YAHOO_STYLE_URLOFF );
		chatOut( GlobalSessionHook, buf );
	} else {
		chatOut( GlobalSessionHook,
			YAHOO_STYLE_BOLDON "Gyach" YAHOO_STYLE_BOLDOFF
			" v" VERSION " "
			YAHOO_STYLE_URLON
			GYACH_URL
			YAHOO_STYLE_URLOFF
			);
	}
}

void cmd_ignore( char *user ) {
	ignore_toggle( user );
}

void cmd_unignore( char *user ) {
	if ( ignore_check( user )) {
		ignore_toggle( user );
	}
}

void cmd_quit() {
	GlobalSessionHook->quit = 1;
}

void cmd_exec( char *cmd ) {
	char data[128];
	char buf[513] = "";
	char *ptr;
	FILE *fp;

#ifdef FORK_EXEC
	if ( ! fork()) {
		/* we are in child */
#endif

		fp = popen( cmd, "r" );

		if ( fp ) {
			strcat( buf, "`" );
			strcat( buf, cmd );
			strcat( buf, "` ===>\n" );

			while( fgets( data, 128, fp )) {
				if (( strlen( buf ) + strlen( data )) < 512 )
					strcat( buf, data );
			}
			pclose( fp );

			ptr = buf + strlen( buf );
			ptr--;
			while(( *ptr == '\n' ) || ( *ptr == '\r' )) {
				*ptr = '\0';
				ptr--;
			}

			send_yahoo_text_command( "say say\001", buf );
		}

#ifdef FORK_EXEC
		_exit(0);
	}
#endif
}

int try_command( char *input ) {
	char *action = NULL;
	char *args = NULL;
	char tmp_input[ 256 ];

	strcpy( tmp_input, input );
	action = tmp_input + 1;

	action = skip_whitespace( action ); /* find head of command */
	args = action;
	args = find_whitespace( args ); /* find whitespace after command */
	if ( *args ) {
		*args = '\0';
		args++;
		args = skip_whitespace( args ); /* find text after whitespace */
	}

	if ( action[0] == '\0' )
		return( 0 );

	if ( !strcmp( "join", action )) {
		cmd_join( args );
	} else if ( !strcmp( "goto", action )) {
		cmd_goto( args );
	} else if ( !strcmp( "follow", action )) {
		cmd_follow( args );
	} else if (( !strcmp( "call", action )) ||
			   ( !strcmp( "invite", action ))) {
		cmd_invite( args );
	} else if (( !strcmp( "cls", action )) ||
			   ( !strcmp( "clear", action ))) {
		cmd_clear();
	} else if ( !strcmp( "tell", action )) {
		cmd_tell( args );
	} else if ( !strcmp( "away", action )) {
		cmd_away( args );
	} else if ( !strcmp( "autoaway", action )) {
		cmd_auto_away( args );
	} else if ( !strcmp( "think", action )) {
		cmd_think( args );
	} else if ( !strcmp( "roll", action )) {
		cmd_roll( args );
	} else if ( !strcmp( "ignore", action )) {
		cmd_ignore( args );
	} else if ( !strcmp( "unignore", action )) {
		cmd_unignore( args );
	} else if ( !strcmp( "help", action )) {
		cmd_help( args );
	} else if (( !strcmp( "url", action )) ||
			   ( !strcmp( "ver", action ))) {
		cmd_url( args );
	} else if (( !strcmp( "quit", action )) ||
			   ( !strcmp( "exit", action )) ||
			   ( !strcmp( "logoff", action )) ||
			   ( !strcmp( "logout", action ))) {
		cmd_quit();
	} else if (( !strcmp( "exec", action )) ||
			   ( !strcmp( "x", action ))) {
		cmd_exec( args );
	} else {
		return( 0 );
	}

	return( 1 );
}

