/*
 * gyach.c
 *
 * Copyright 2000, Chris Pinkham.
 * All Rights Reserved.
 */

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <gdk_imlib.h>
#include <regex.h>


#include "parsecfg.h"
#include "gyach.h"
#include "images.h"
#include "yahoolib.h"

#include "interface.h"
#include "support.h"
#include "nix.h"

int debug_level = 00;
int debug_packets = 0;

GtkWidget * login_window;
GtkWidget * chat_window;
GtkWidget * chat_text;
GtkWidget * chat_entry;
GtkWidget * chat_emotions;
GtkWidget * chat_users;
GtkWidget * chat_user_menu;
GtkWidget * emote_menu;
GtkWidget * emote_window;
GtkWidget * pm_window;
GtkWidget * pm_entry;
GtkWidget * pm_user;
GtkWidget * chat_username;
GtkWidget * chat_password;
GtkWidget * chat_roomname;
GtkWidget * profile_window = NULL;
GtkWidget * about_box = NULL;
GtkWidget * setup_window = NULL;

int  emotion_count;
char *user_selected = NULL;
int  emote_selected = -1;
char *follow_user = NULL;
char *capture_filename = NULL;
FILE *capture_fp = NULL;

int my_status = 0;
time_t auto_away_time;

int  use_blended_colors = 0;
int  use_my_colors = 0;
int  use_my_font = 0;
int  def_user = -1;
int  auto_away = 15;
char *username0 = NULL;
char *username1 = NULL;
char *username2 = NULL;
char *username3 = NULL;
char *username4 = NULL;
char *password0 = NULL;
char *password1 = NULL;
char *password2 = NULL;
char *password3 = NULL;
char *password4 = NULL;
char *def_room  = NULL;
char *fav_room0 = NULL;
char *fav_room1 = NULL;
char *fav_room2 = NULL;
char *fav_room3 = NULL;
char *fav_room4 = NULL;
char *fav_room5 = NULL;
char *fav_room6 = NULL;
char *fav_room7 = NULL;
char *fav_room8 = NULL;
char *fav_room9 = NULL;

char cfg_filename[ 1024 ] = "";
char cfg_dir[1024] = "";

char username[41] = "";
char password[41] = "";
char roomname[129] = "";

int connected = 0;
int valid_user = 0;

char *ignore_users[2048];
char *auto_ignore[2048];
regex_t ai_regex[2048];
int ignore_count = 0;
int auto_ignore_count = 0;

SESSION *GlobalSessionHook;

void append_to_textbox(GtkWidget * window, GtkWidget * textbox, char *text);

cfgStruct cfg[] = {
	/* parameter  type address of variable */
	{ "use_blended_colors",		CFG_INT,		&use_blended_colors },
	{ "use_my_colors",			CFG_INT,		&use_my_colors },
	{ "use_my_font",			CFG_INT,		&use_my_font },
	{ "debug_packets",			CFG_INT,		&debug_packets },
	{ "def_user",				CFG_INT,		&def_user },
	{ "auto_away",				CFG_INT,		&auto_away },
	{ "username0",				CFG_STRING,		&username0 },
	{ "username1",				CFG_STRING,		&username1 },
	{ "username2",				CFG_STRING,		&username2 },
	{ "username3",				CFG_STRING,		&username3 },
	{ "username4",				CFG_STRING,		&username4 },
	{ "password0",				CFG_STRING,		&password0 },
	{ "password1",				CFG_STRING,		&password1 },
	{ "password2",				CFG_STRING,		&password2 },
	{ "password3",				CFG_STRING,		&password3 },
	{ "password4",				CFG_STRING,		&password4 },
	{ "def_room",				CFG_STRING,		&def_room },
	{ "fav_room0",				CFG_STRING,		&fav_room0 },
	{ "fav_room1",				CFG_STRING,		&fav_room1 },
	{ "fav_room2",				CFG_STRING,		&fav_room2 },
	{ "fav_room3",				CFG_STRING,		&fav_room3 },
	{ "fav_room4",				CFG_STRING,		&fav_room4 },
	{ "fav_room5",				CFG_STRING,		&fav_room5 },
	{ "fav_room6",				CFG_STRING,		&fav_room6 },
	{ "fav_room7",				CFG_STRING,		&fav_room7 },
	{ "fav_room8",				CFG_STRING,		&fav_room8 },
	{ "fav_room9",				CFG_STRING,		&fav_room9 },
	{ NULL,						CFG_END,		NULL }
};

/* fixme */
void no_home_dir_message() {
	/* print out a message or open a window or something */
}

void gyach_init() {
	read_config();
	ignore_load();

	auto_away_time = time(NULL) + ( auto_away * 60 );
}

void read_config() {
	struct stat sbuf;

	if ( ! getenv( "HOME" )) {
		no_home_dir_message();
		return;
	}

	if ( cfg_filename[ 0 ] == '\0' )
		sprintf( cfg_filename, "%s/.gyach/gyachrc", getenv( "HOME" ));

	if ( stat( cfg_filename, &sbuf ))
		write_config();

	cfgParse( cfg_filename, cfg, CFG_SIMPLE );

	if ( debug_level >= 20 ) {
		printf( "%s\n", cfg_filename );
		printf( "  use_blended_colors = %d\n", use_blended_colors );
		printf( "  use_my_colors = %d\n", use_my_colors );
		printf( "  use_my_font = %d\n", use_my_font );
		printf( "  def_user = %d\n", def_user );
		printf( "  username0 = '%s'\n", username0 );
		printf( "  username1 = '%s'\n", username1 );
		printf( "  username2 = '%s'\n", username2 );
		printf( "  username3 = '%s'\n", username3 );
		printf( "  username4 = '%s'\n", username4 );
		printf( "  def_room = '%s'\n", def_room );
	}
}

void write_config() {
	FILE *gyachrc;
	struct stat sbuf;

	if ( ! getenv( "HOME" )) {
		no_home_dir_message();
		return;
	}

	if ( cfg_dir[ 0 ] == '\0' )
		sprintf( cfg_dir, "%s/.gyach", getenv( "HOME" ));

	if ( stat( cfg_dir, &sbuf ))
		mkdir( cfg_dir, 0700 );

	if ( cfg_filename[ 0 ] == '\0' )
		sprintf( cfg_filename, "%s/.gyach/gyachrc", getenv( "HOME" ));

	gyachrc = fopen( cfg_filename, "w" );

	if ( gyachrc ) {
		if (username0) fprintf( gyachrc, "username0 = \"%s\"\n", username0 );
		if (username1) fprintf( gyachrc, "username1 = \"%s\"\n", username1 );
		if (username2) fprintf( gyachrc, "username2 = \"%s\"\n", username2 );
		if (username3) fprintf( gyachrc, "username3 = \"%s\"\n", username3 );
		if (username4) fprintf( gyachrc, "username4 = \"%s\"\n", username4 );
		if (password0) fprintf( gyachrc, "password0 = \"%s\"\n", password0 );
		if (password1) fprintf( gyachrc, "password1 = \"%s\"\n", password1 );
		if (password2) fprintf( gyachrc, "password2 = \"%s\"\n", password2 );
		if (password3) fprintf( gyachrc, "password3 = \"%s\"\n", password3 );
		if (password4) fprintf( gyachrc, "password4 = \"%s\"\n", password4 );
		if (fav_room0) fprintf( gyachrc, "fav_room0 = \"%s\"\n", fav_room0 );
		if (fav_room1) fprintf( gyachrc, "fav_room1 = \"%s\"\n", fav_room1 );
		if (fav_room2) fprintf( gyachrc, "fav_room2 = \"%s\"\n", fav_room2 );
		if (fav_room3) fprintf( gyachrc, "fav_room3 = \"%s\"\n", fav_room3 );
		if (fav_room4) fprintf( gyachrc, "fav_room4 = \"%s\"\n", fav_room4 );
		if (fav_room5) fprintf( gyachrc, "fav_room5 = \"%s\"\n", fav_room5 );
		if (fav_room6) fprintf( gyachrc, "fav_room6 = \"%s\"\n", fav_room6 );
		if (fav_room7) fprintf( gyachrc, "fav_room7 = \"%s\"\n", fav_room7 );
		if (fav_room8) fprintf( gyachrc, "fav_room8 = \"%s\"\n", fav_room8 );
		if (fav_room9) fprintf( gyachrc, "fav_room9 = \"%s\"\n", fav_room9 );

		fprintf( gyachrc, "use_blended_colors = %d\n", use_blended_colors );
		fprintf( gyachrc, "use_my_colors = %d\n", use_my_colors );
		fprintf( gyachrc, "use_my_font = %d\n", use_my_font );
		fprintf( gyachrc, "debug_packets = %d\n", debug_packets );
		fprintf( gyachrc, "auto_away = %d\n", auto_away );

		fclose( gyachrc );
	}
}

/*
 * emotion datafile fmt is "emotion]everyone|individual" similar to
 * CheetaChat but does NOT support %s for user's sex, only %N for other user's
 * name.
 */
void read_emotions() {
	gchar *listitem[3];
	FILE *fp;
	char line[1024];
	char *emotion = NULL;
	char *em_everyone = NULL;
	char *em_user = NULL;
	char *ptr = NULL;
	char emot_filename[1024] = "";
	int row;

	gtk_clist_set_sort_column(GTK_CLIST(chat_emotions), 0 );
	gtk_clist_set_auto_sort(GTK_CLIST(chat_emotions), 1 );

	gtk_clist_freeze(GTK_CLIST(chat_emotions));
	
	listitem[0] = emotion;
	listitem[1] = em_everyone;
	listitem[2] = em_user;
	emotion = line;

	if ( emot_filename[ 0 ] == '\0' )
		sprintf( emot_filename, "%s/.gyach/emotions.dat",
			getenv( "HOME" ));

	fp = fopen( emot_filename, "r" );

	emotion_count = 0;

	if ( fp ) {
		while( fgets( line, 1023, fp )) {
			if ( ! strchr( line, ']' ))	continue;

			while(( line[strlen(line)-1] == '\r' ) ||
				  ( line[strlen(line)-1] == '\n' )) {
				line[strlen(line)-1] = '\0';
			}

			emotion_count++;

			ptr = line;
			while( *ptr != ']' ) {
				*ptr = tolower( *ptr );
				ptr++;
			}

			line[0] = toupper( line[0] );
			ptr = strchr( line, ']' );
			*ptr = '\0';

			em_everyone = ptr + 1;
			ptr++;

			ptr = strchr( em_everyone, '|' );
			*ptr = '\0';
			em_user = ptr + 1;

			listitem[0] = emotion;
			row = gtk_clist_insert(GTK_CLIST(chat_emotions), 0, listitem );
			gtk_clist_set_text(GTK_CLIST(chat_emotions), row, 1, em_everyone );
			gtk_clist_set_text(GTK_CLIST(chat_emotions), row, 2, em_user );
		}
		fclose( fp );
	} else {
		/* no emotion file so set defaults */
	}

	gtk_clist_thaw(GTK_CLIST(chat_emotions));
}

void save_emotions() {
	gchar *em_name;
	gchar *em_everyone;
	gchar *em_user;
	char emot_filename[1024] = "";
	int i = 0;
	FILE *fp;

	if ( emot_filename[ 0 ] == '\0' )
		sprintf( emot_filename, "%s/.gyach/emotions.dat",
			getenv( "HOME" ));

	fp = fopen( emot_filename, "wb" );

	if ( fp ) {
		while(1) {
			if ( !gtk_clist_get_text(GTK_CLIST(chat_emotions), i, 0,&em_name)) {
				break;
			}

			gtk_clist_get_text(GTK_CLIST(chat_emotions), i, 1, &em_everyone );
			gtk_clist_get_text(GTK_CLIST(chat_emotions), i, 2, &em_user );

			fprintf( fp, "%s]%s|%s\n", em_name, em_everyone, em_user );
			
			i++;
		}
		fclose( fp );
	}
}

void strip_html_tags( char *str ) {
	char *to, *from;

	to = str;
	from = str;

	while( *from ) {
		if ( *from == '<' ) {
			from++;
			while(( *from != '>' ) &&
				  ( *from != '<' )) {
				from++;
			}
			from++;
		} else {
			fflush( stdout );
			*to = *from;
			to++;
			from++;
		}
	}
	*to = *from;

	to = str;
	from = str;
	while( *from ) {
		if ( *from == '\r' ) {
			from++;
		} else if ( *from == '&' ) {
			if ( *(from+1) == '#' ) {
				while( *from != ';' ) {
					from++;
				}
			} else if (( *(from+1) == 'n' ) &&
					   ( *(from+2) == 'b' ) &&
					   ( *(from+3) == 's' ) &&
					   ( *(from+4) == 'p' ) &&
					   ( *(from+5) == ';' )) {
				from += 5;
				*to = ' ';
				to++;
			}
			from++;
		} else {
			*to = *from;
			to++;
			from++;
		}
	}
	*to = *from;

	to = str;
	from = str;
	while( *from ) {
		if (( *from == '\n' ) &&
			( *(from+1) == '\n' )) {
		} else {
			*to = *from;
			to++;
		}
		from++;
	}
	*to = *from;
}

char *download_image( char *image_url ) {
	static char tmp_filename[1025] = "/tmp/gyach.img";
	int sock;
	struct sockaddr_in sa;
	struct hostent *hinfo;
	char url[1025];
	char result[65535];
	char tmp[65535];
	char *ptr;
	int nr = 0, tr = 0;
	int ifp;

	char host[75] = "";

	if ( strncmp( image_url, "http://", 7 ))
		return "";

	strncpy( host, image_url + 7, 74 );
	host[74] = '\0';
	ptr = strchr( host, '/' );
	*ptr = '\0';

	ptr = strchr( image_url + 7, '/' );

	hinfo = gethostbyname( host );

	if ( ! hinfo || ( hinfo->h_addrtype != AF_INET )) {
		/* fixme, show an error box */
		return "";
	}

	memset( &sa, 0, sizeof(sa));
	memmove((void*)&sa.sin_addr.s_addr, hinfo->h_addr, hinfo->h_length );

	sa.sin_family = AF_INET;
	sa.sin_port = htons( 80 );

	if (( sock = socket( AF_INET, SOCK_STREAM, 6 )) == -1 ) {
		/* fixme, error message */
		if ( debug_level > 10 ) {
			printf( "error in socket()\n" );
		}
		return "";
	} else {
		if ( connect( sock, (struct sockaddr*)&sa, sizeof(sa)) == -1 ) {
			/* fixme, error message */
			if ( debug_level > 10 ) {
				printf( "error in connect()\n" );
			}
			close(sock);
			return "";
		}
	}

	snprintf( url, 1024,
		"GET %s HTTP/1.0\r\n"
		"Accept: text/html\r\n"
		"Accept: text/plain\r\n"
		"User-Agent: gyach/?.?\r\n"
		"\r\n"
		"\r\n", ptr );

	write( sock, url, strlen( url ));

	result[0] = '\0';

	tr = 0;
	nr = read( sock, tmp, 65535 );
	while( nr > 0 ) {
		tmp[nr] = '\0';
		memcpy( result + tr, tmp, nr );
		tr += nr;
		nr = read( sock, tmp, 65535 - tr );
	}

	close( sock );

	ptr = strstr( result, "\r\n\r\n" );
	ptr += 4;

	if ( strstr( result, "Not Found" ))
		return( "" );

	ifp = open( tmp_filename, O_CREAT | O_TRUNC| O_WRONLY, 0600 );
	write( ifp, ptr, tr - ( ptr - result ));
	close( ifp );

	return tmp_filename;
}

void display_profile( char *user ) {
	GdkImlibImage *orig_image;
	GdkImlibImage *scaled_image;

	int sock;
	struct sockaddr_in sa;
	struct hostent *hinfo;
	char url[1025];
	char result[32769];
	char tmp[32769];
	int nr = 0, tr = 0;

	char *ptr;
	char pic_url[129] = "";
	char last_updated[21] = "";
	char email[129] = "";
	char realname[51] = "";
	char location[51] = "";
	char age[5] = "";
	char married[21] = "";
	char sex[11] = "";
	char occupation[51] = "";
	char online_now[11] = "";
	char homepage[129] = "";
	char coollink1[129] = "";
	char coollink2[129] = "";
	char coollink3[129] = "";

	GtkWidget	*tmp_widget;

	hinfo = gethostbyname( "profiles.yahoo.com" );

	if ( ! hinfo || ( hinfo->h_addrtype != AF_INET )) {
		/* fixme, show an error box */
		return;
	}

	memset( &sa, 0, sizeof(sa));
	memmove((void*)&sa.sin_addr.s_addr, hinfo->h_addr, hinfo->h_length );

	sa.sin_family = AF_INET;
	sa.sin_port = htons( 80 );

	if (( sock = socket( AF_INET, SOCK_STREAM, 6 )) == -1 ) {
		/* fixme, error message */
		if ( debug_level > 10 ) {
			printf( "error in socket()\n" );
		}
		return;
	} else {
		if ( connect( sock, (struct sockaddr*)&sa, sizeof(sa)) == -1 ) {
			/* fixme, error message */
			if ( debug_level > 10 ) {
				printf( "error in connect()\n" );
			}
			close(sock);
			return; 
		}
	}

	snprintf( url, 1024,
		"GET /%s HTTP/1.0\r\n"
		"Accept: text/html\r\n"
		"Accept: text/plain\r\n"
		"User-Agent: gyach/?.?\r\n"
		"\r\n"
		"\r\n", user );

	write( sock, url, strlen( url ));

	result[0] = '\0';

	tr = 0;
	nr = read( sock, tmp, 32768 );
	while( nr > 0 ) {
		tmp[nr] = '\0';
		strcat( result, tmp );
		tr += nr;
		nr = read( sock, tmp, 32768 - tr );
	}

	/* find the url to their pic */
	sprintf( tmp, "alt=\"%s\"", user );
	ptr = tmp;
	while( *ptr )
		*(ptr++) = tolower( *ptr );
	ptr = strstr( result, tmp );
	while(( *(ptr-5) != 's' ) || ( *(ptr-4) != 'r' ) || ( *(ptr-3) != 'c' ) ||
		  ( *(ptr-2) != '=' ) || ( *(ptr-1) != '"' )) {
		ptr--;
	}
	/* now we are at the beginning of the url, so strncpy some */
	strncpy( pic_url, ptr, 128 );
	ptr = strchr( pic_url, '"' );
	*ptr = '\0';

	/* strip html to make the rest easier */
	strip_html_tags( result );

	/* attempt to find some strings */
	ptr = strstr( result, "Last Updated:" );
	if ( ptr ) {
		ptr = strchr( ptr, ':' );
		while( *ptr != '\n' )
			ptr++;
		while( *ptr == '\n' ) 
			ptr++;
		strncpy( last_updated, ptr, 20 );
		ptr = strchr( last_updated, '\n' );
		*ptr = '\0';
	}

	ptr = strstr( result, "My Email" );
	if ( ptr ) {
		while( *ptr != '\n' )
			ptr++;
		while( *ptr == '\n' ) 
			ptr++;
		strncpy( email, ptr, 128 );
		ptr = strchr( email, '\n' );
		*ptr = '\0';
	}

	ptr = strstr( result, "Real Name:" );
	if ( ptr ) {
		while( *ptr != '\n' )
			ptr++;
		while( *ptr == '\n' ) 
			ptr++;
		strncpy( realname, ptr, 50 );
		ptr = strchr( realname, '\n' );
		*ptr = '\0';

		if ( strchr( realname, ':' ))
			realname[0] = '\0';
	}

	ptr = strstr( result, "Location:" );
	if ( ptr ) {
		while( *ptr != '\n' )
			ptr++;
		while( *ptr == '\n' ) 
			ptr++;
		strncpy( location, ptr, 50 );
		ptr = strchr( location, '\n' );
		*ptr = '\0';

		if ( strchr( location, ':' ))
			location[0] = '\0';
	}

	ptr = strstr( result, "Age:" );
	if ( ptr ) {
		while( *ptr != '\n' )
			ptr++;
		while( *ptr == '\n' ) 
			ptr++;
		strncpy( age, ptr, 4 );
		ptr = strchr( age, '\n' );
		*ptr = '\0';

		if ( strchr( age, ':' ))
			age[0] = '\0';
	}

	ptr = strstr( result, "Marital Status:" );
	if ( ptr ) {
		while( *ptr != '\n' )
			ptr++;
		while( *ptr == '\n' ) 
			ptr++;
		strncpy( married, ptr, 20 );
		ptr = strchr( married, '\n' );
		*ptr = '\0';

		if ( strchr( married, ':' ))
			married[0] = '\0';
	}

	ptr = strstr( result, "Sex:" );
	if ( ptr ) {
		while( *ptr != '\n' )
			ptr++;
		while( *ptr == '\n' ) 
			ptr++;
		strncpy( sex, ptr, 10 );
		ptr = strchr( sex, '\n' );
		*ptr = '\0';

		if ( strchr( sex, ':' ))
			sex[0] = '\0';
	}

	ptr = strstr( result, "Occupation:" );
	if ( ptr ) {
		while( *ptr != '\n' )
			ptr++;
		while( *ptr == '\n' ) 
			ptr++;
		strncpy( occupation, ptr, 50 );
		ptr = strchr( occupation, '\n' );
		*ptr = '\0';

		if ( strchr( occupation, ':' ))
			occupation[0] = '\0';
	}

	ptr = strstr( result, "I'm online now!" );
	if ( ptr ) {
		strcpy( online_now, "Online now!" );
	} else {
		strcpy( online_now, "OFFline" );
	}

	if ( debug_level >= 10 ) {
		printf( "profile info for %s:\n", user );
		printf( "Picture         : '%s'\n", pic_url );
		printf( "Last Updated    : '%s'\n", last_updated );
		printf( "Email           : '%s'\n", email );
		printf( "Real Name       : '%s'\n", realname );
		printf( "Location        : '%s'\n", location );
		printf( "Age             : '%s'\n", age );
		printf( "Married?        : '%s'\n", married );
		printf( "Sex             : '%s'\n", sex );
		printf( "Occupation      : '%s'\n", occupation );
		printf( "Online Now      : '%s'\n", online_now );
		printf( "Home Page       : '%s'\n", homepage );
		printf( "Cool Link #1    : '%s'\n", coollink1 );
		printf( "Cool Link #2    : '%s'\n", coollink2 );
		printf( "Cool Link #3    : '%s'\n", coollink3 );
	}

	close(sock);


	if ( ! profile_window ) {
		profile_window = create_profile_window();
	}

	gtk_widget_show( profile_window );

	tmp_widget = lookup_widget( profile_window, "username" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), user );
	tmp_widget = lookup_widget( profile_window, "last_updated" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), last_updated );
	tmp_widget = lookup_widget( profile_window, "realname" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), realname );
	tmp_widget = lookup_widget( profile_window, "email" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), email );
	tmp_widget = lookup_widget( profile_window, "location" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), location );
	tmp_widget = lookup_widget( profile_window, "age" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), age );
	tmp_widget = lookup_widget( profile_window, "married" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), married );
	tmp_widget = lookup_widget( profile_window, "sex" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), sex );
	tmp_widget = lookup_widget( profile_window, "occupation" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), occupation );
	tmp_widget = lookup_widget( profile_window, "online_now" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), online_now );
	tmp_widget = lookup_widget( profile_window, "homepage" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), homepage );
	tmp_widget = lookup_widget( profile_window, "coollink1" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), coollink1 );
	tmp_widget = lookup_widget( profile_window, "coollink2" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), coollink2 );
	tmp_widget = lookup_widget( profile_window, "coollink3" );
	gtk_label_set_text( GTK_LABEL(tmp_widget), coollink3 );

	tmp_widget = lookup_widget( profile_window, "user_picture" );

	ptr = download_image( pic_url );

/* for testing purposes
	system( "xv /tmp/gyach.img" );
*/

	orig_image = gdk_imlib_load_image( ptr );

	scaled_image = gdk_imlib_clone_scaled_image( orig_image, 150, 150 );
	gdk_imlib_destroy_image( orig_image );
	unlink( ptr );

	gdk_imlib_save_image_to_ppm( scaled_image, ptr );
	gdk_imlib_destroy_image( scaled_image );

	gdk_imlib_load_file_to_pixmap( ptr, &(GTK_PIXMAP(tmp_widget)->pixmap),
		&(GTK_PIXMAP(tmp_widget)->mask) );

	unlink( ptr );
}

void display_url( char *url ) {
	char cmd[1025];

	if ( strlen( url ) == 0 )
		return;

	/* quick sanity check, fixme */
	if (( strchr( url, ';' )) ||
		( strchr( url, '`' ))) {
		return;
	}

	snprintf( cmd, 1024,
		"netscape -noraise -remote openURL\\(%s,yahmini\\)", url );

	system( url );
}

void lower_str( char *str ) {
	char *ptr = str;

	while( *ptr ) {
		*(ptr++) = tolower( *ptr );
	}
}

void edit_emote() {
	gchar *listitem[3];
	gchar emotion[128] = "";
	gchar em_everyone[128] = "";
	gchar em_user[128] = "";
	GtkWidget *emote_name;
	GtkWidget *emote_everyone;
	GtkWidget *emote_user;

	listitem[0] = emotion;
	listitem[1] = em_everyone;
	listitem[2] = em_user;

	if ( emote_selected > -1 ) {
		gtk_clist_get_text( GTK_CLIST(chat_emotions), emote_selected, 0,
			listitem );
		gtk_clist_get_text( GTK_CLIST(chat_emotions), emote_selected, 1,
			listitem+1 );
		gtk_clist_get_text( GTK_CLIST(chat_emotions), emote_selected, 2,
			listitem+2 );
	}

    if ( ! emote_window ) {
		emote_window = create_emote_window();
		gtk_widget_show( emote_window );
	}

	emote_name = lookup_widget( emote_window, "emote_name" );
	emote_everyone = lookup_widget( emote_window, "emote_everyone" );
	emote_user = lookup_widget( emote_window, "emote_user" );

	gtk_entry_set_text( GTK_ENTRY(emote_name), listitem[0] );
	gtk_entry_set_text( GTK_ENTRY(emote_everyone), listitem[1] );
	gtk_entry_set_text( GTK_ENTRY(emote_user), listitem[2] );
}

int find_user_row( char *user ) {
	char tmp_user[129];
	char tmp_list[129];
	char *listitem[1];
	int i;
	int r;

	strcpy( tmp_user, user );
	lower_str( tmp_user );

	r = -1;
	for( i = 0 ; i < GTK_CLIST(chat_users)->rows && r == -1; i++ ) {
		gtk_clist_get_text( GTK_CLIST(chat_users), i, 1, listitem );
		strcpy( tmp_list, listitem[0] );
		lower_str( tmp_list );
		if ( ! strcmp( tmp_list, tmp_user ))
			r = i;
	}

	return( r );
}

int chars_match( char *str1, char *str2 ) {
	int i = 0;

	/* if the 2nd string is longer then it's can't be the same word */
	if ( strlen( str2 ) > strlen( str1 ) )
		return( 0 );

	while(( *str1 ) && ( *str2 ) && ( *str1 == *str2 )) {
		i++;
		str1++;
		str2++;
	}

	return( i );
}

/*
 * fixme, this function needs cleaning up probably
 */
char *find_user_best_match( char *part ) {
	static char best_match[129] = "";
	static char last_part[129] = "";
	static int  last_match = 0;
	char tmp_user[129];
	char tmp_list[129];
	char *listitem[1];
	int bm = last_match;


	if ( ! part[0] ) {
		best_match[0] = '\0';
		last_part[0] = '\0';
		return( best_match );
	}

	if (( ! last_part[0] ) || strncmp( part, last_part, strlen( last_part ))) {
		best_match[0] = '\0';
		last_match = 0;
		strcpy( last_part, part );
	} else {
		strcpy( part, last_part );
		last_match++;
	}

	if ( last_match >= GTK_CLIST(chat_users)->rows )
		last_match = 0;

	strcpy( tmp_user, part );
	lower_str( tmp_user );

	gtk_clist_get_text( GTK_CLIST(chat_users), last_match, 1, listitem );
	strcpy( tmp_list, listitem[0] );
	lower_str( tmp_list );

	while(( strncmp( tmp_list, part, strlen( part ))) &&
		  ( last_match < GTK_CLIST(chat_users)->rows )) {
		last_match++;
		gtk_clist_get_text( GTK_CLIST(chat_users), last_match, 1, listitem );
		strcpy( tmp_list, listitem[0] );
		lower_str( tmp_list );
	}

	/* did we hit the end of the list after starting at the beginning */
	/* if so, return */
	if (( last_match >= GTK_CLIST(chat_users)->rows ) &&
		( bm == 0 )) {
		strcpy( best_match, part );
		last_match = 0;
		return( best_match );
	} 

	/* if we don't match, try to start from the beginning again */
	if ( strncmp( tmp_list, part, strlen( part ))) {
		last_match = 0;
		gtk_clist_get_text( GTK_CLIST(chat_users), last_match, 1, listitem );
		strcpy( tmp_list, listitem[0] );
		lower_str( tmp_list );

		while(( strncmp( tmp_list, part, strlen( part ))) &&
			  ( last_match < bm )) {
			last_match++;
			gtk_clist_get_text( GTK_CLIST(chat_users), last_match, 1,
				listitem );
			strcpy( tmp_list, listitem[0] );
			lower_str( tmp_list );
		}
	}

	if ( strncmp( tmp_list, part, strlen( part ))) {
		strcpy( best_match, part );
		last_match = 0;
	} else {
		strcpy( best_match, tmp_list );
	}
		
	return( best_match );
}

void ignore_add( char *user ) {
	char tmp_user[129];
	int i;

	if ( ! user )
		return;

	strcpy( tmp_user, user );
	lower_str( tmp_user );

	for( i = 0; i < ignore_count; i++ ) {
		if ( !strcmp( tmp_user, ignore_users[i] )) {
			break;
		}
	}

	if ( i >= ignore_count ) {
		ignore_users[ignore_count++] = strdup( tmp_user );
	}
}

void ignore_remove( char *user ) {
	char tmp_user[129];
	int i;

	if ( ! user )
		return;

	strcpy( tmp_user, user );
	lower_str( tmp_user );

	for ( i = 0; i < ignore_count; i++ ) {
		if ( ! strcmp( ignore_users[i], tmp_user )) {
			free( ignore_users[i] );
			ignore_users[i] = ignore_users[ignore_count-1];
			ignore_count--;
		}
	}
}

int ignore_check( char *user ) {
	char tmp_user[129];
	int i;

	if ( ! user )
		return( 0 );

	strcpy( tmp_user, user );
	lower_str( tmp_user );

	if ( ! ignore_count )
		return( 0 );

	for ( i = 0; i < ignore_count; i++ ) {
		if ( ! strcmp( ignore_users[i], tmp_user )) {
			return( 1 );
		}
	}

	return( 0 );
}

int auto_ignore_check( char *str ) {
	int i;
	

	if ( ! str )
		return( 0 );

	if ( ! auto_ignore_count )
		return( 0 );

	for ( i = 0; i < auto_ignore_count; i++ ) {
		if ( ! regexec( &ai_regex[i], str, 0, NULL, 0 )) {
			return( 1 );
		}
	}

	return( 0 );
}

void ignore_load() {
	char if_filename[1024];
	char buf[129];
	FILE *fp;
	int i;

	if ( cfg_dir[ 0 ] == '\0' )
		sprintf( cfg_dir, "%s/.gyach", getenv( "HOME" ));

	sprintf( if_filename, "%s/ignore.list", cfg_dir );

	fp = fopen( if_filename, "r" );

	if ( fp ) {
		/* clear the list if there already is one */
		if ( ignore_count ) {
			for( i = 0; i < ignore_count ; i++ ) {
				free( ignore_users[i] );
			}
			ignore_count = 0;
		}

		/* load the new list */
		while( fgets( buf, 128, fp )) {
			/* chop the newline */
			buf[strlen(buf)-1] = '\0';

			ignore_users[ignore_count++] = strdup( buf );
		}

		fclose( fp );
	}

	sprintf( if_filename, "%s/ignore.regex", cfg_dir );

	fp = fopen( if_filename, "r" );

	if ( fp ) {
		/* clear the list if there already is one */
		if ( auto_ignore_count ) {
			for( i = 0; i < auto_ignore_count ; i++ ) {
				free( auto_ignore[i] );
			}
			auto_ignore_count = 0;
		}

		/* load the new list */
		while( fgets( buf, 128, fp )) {
			/* chop the newline */
			buf[strlen(buf)-1] = '\0';

			regcomp( &ai_regex[auto_ignore_count], buf,
				REG_ICASE | REG_NOSUB );
			auto_ignore[auto_ignore_count++] = strdup( buf );
		}

		fclose( fp );
	}
}

void ignore_save() {
	char if_filename[1024];
	FILE *fp;
	int i;

	if ( cfg_dir[ 0 ] == '\0' )
		sprintf( cfg_dir, "%s/.gyach", getenv( "HOME" ));

	sprintf( if_filename, "%s/ignore.list", cfg_dir );

	fp = fopen( if_filename, "w" );

	if ( fp ) {
		for( i = 0; i < ignore_count; i++ ) {
			fprintf( fp, "%s\n", ignore_users[i] );
		}

		fclose( fp );
	}
}

void ignore_toggle( char *user ) {
	int r;
	int i;
	GdkPixmap *st_pixmap;
	GdkBitmap *st_mask;
	char buf[129] = "";

	r = find_user_row( user );

	if ( user[0] ) {
		if ( ignore_check( user )) {
			ignore_remove( user );
			if ( r >= 0 ) {
				gdk_imlib_data_to_pixmap( pixmap_status_here, &st_pixmap,
					&st_mask );
			}
			sprintf( buf, "*** no longer ignoring %s%s%s ***\n",
				YAHOO_COLOR_BLUE, user, YAHOO_COLOR_BLACK );
		} else {
			ignore_add( user );
			if ( r >= 0 ) {
				gdk_imlib_data_to_pixmap( pixmap_status_ignore, &st_pixmap,
					&st_mask );
			}
			sprintf( buf, "*** ignoring %s%s%s ***\n",
				YAHOO_COLOR_BLUE, user, YAHOO_COLOR_BLACK );
		}
		append_to_textbox( chat_window, chat_text, buf );
	} else {
		append_to_textbox( chat_window, chat_text,
			"*** Ignoring the following users:\n" );
		if ( ignore_count ) {
			for( i = 0; i < ignore_count ; i++ ) {
				sprintf( buf, "\t%s\n", ignore_users[i] );
				append_to_textbox( chat_window, chat_text, buf );
			}
			sprintf( buf, "\tIgnoring %d users.\n", ignore_count );
			append_to_textbox( chat_window, chat_text, buf );
		} else {
			append_to_textbox( chat_window, chat_text, "\tNo Users Ignored\n" );
		}
	}

	ignore_save();

	if ( r < 0 )
		return;

	gtk_clist_set_pixmap(GTK_CLIST(chat_users), r, 0, st_pixmap, st_mask );
}

/*
 * dump a yin packet out to stdout for testing purposes
 */
void dump_packet() {
	int i = 0;
	int x = 0;
	int y = 0;
	char *ptr;

	printf( "type = 0x%x, size = %d\n",
		GlobalSessionHook->yin.type,
		GlobalSessionHook->yin.size );

	ptr = GlobalSessionHook->yin.data;
	while( i < GlobalSessionHook->yin.size ) {
		if ( x < 16 ) {
			printf( "%02x ", *(ptr+i) & 0xFF );
			x++;
			i++;
		} else {
			printf( "   " );
			for( ; x > 0 ; x-- ) {
				if ( isgraph( *(ptr+i-x) ) || *(ptr+i-x)  == ' ' ) {
					printf( "%c", *(ptr+i-x) );
				} else {
					printf( "." );
				}
			}

			printf( "\n" );
			x = 0;
		}
	}
	for( y = x; y < 16 ; y++ ) {
		printf( "   " );
	}
	printf( "   " );
	for( ; x > 0 ; x-- ) {
		if ( isgraph( *(ptr+i-x) ) || *(ptr+i-x)  == ' ' ) {
			printf( "%c", *(ptr+i-x) );
		} else {
			printf( "." );
		}
	}

	printf( "\n" );
}

/*
 * attempt to highlight urls if they aren't already highlighted
 */
void highlight_urls( char *str ) {
	char tmp[4097];
	char *from;
	char *to;
	char *ptr;
	int len = 0;

	/* return if too long */
	if ( strlen( str ) > 4096 )
		return;

	/* return if no urls */
	if ( ! strstr( str, "http://" ))
		return;

	/* return if already marked */
	if ( strstr( str, YAHOO_STYLE_URLON ))
		return;

	memset( tmp, 0, sizeof( tmp ));

	from = str;
	to = tmp;

	ptr = strstr( str, "http://" );

	while( ptr ) {
		len = ptr - from;
		strncpy( to, from, len );
		to += len;
		*to = '\0';

		from += len;

		strcat( to, YAHOO_STYLE_URLON );
		to += strlen( YAHOO_STYLE_URLON );

		while(( *from ) && ( ! isspace( *from ))) {
			*(to++) = *(from++);
		}
		*to = '\0';

		strcat( to, YAHOO_STYLE_URLOFF );
		to += strlen( YAHOO_STYLE_URLOFF );

		ptr = strstr( from, "http://" );
	}

	strcat( to, from );
	to += strlen( from );
	*to = '\0';
	strcpy( str, tmp );
}

/*
 * strip junk (like "<snd" directives out of text
 */
void strip_junk( char *str ) {
	/* fixme */
}

void tab_complete_username() {
	char *text = gtk_entry_get_text( GTK_ENTRY(chat_entry));
	char new[1025];
	char *ptr;

	if ( ! strlen( text ))
		return;

	strcpy( new, text );
	ptr = new + strlen( new ) - 1;

	while(( ptr > new ) && ( ! isspace( *ptr )))
		ptr--;

	if ( isspace( *ptr ))
		ptr++;

	if ( *ptr == '/' ) {
	} else {
		strcpy( ptr, find_user_best_match( ptr ));
	}

	gtk_entry_set_text( GTK_ENTRY(chat_entry), new );
	gtk_entry_set_position( GTK_ENTRY(chat_entry), strlen(new));
}

void capture_text_to_file( char *filename ) {
	FILE *fp;
	int  length;
	int  i;
	gchar ch;
	time_t time_now;

	fp = fopen( filename, "ab" );

	if ( ! fp )
		return;

	length = gtk_text_get_length( GTK_TEXT(chat_text) );

	time_now = time(NULL);

	fprintf( fp, "Captured at: %s", ctime(&time_now) );
	fprintf( fp, "-------------------------------------\n" );

	for( i = 0; i < length; i++ ) {
		ch = GTK_TEXT_INDEX( GTK_TEXT(chat_text), i );
		fwrite( &ch, 1, 1, fp );
	}

	fprintf( fp, "\n" );
	fclose( fp );
}
